
public class CheckerPattern
{
   public static void main(String[] args)
   {
      final byte[][] image = makeTestArray();
      for (int i = 0; i < 64; ++i)
         System.out.println(java.util.Arrays.toString(image[i]));
   }

   public static byte[][] makeTestArray()
   {
      final byte[][] image = new byte[64][64];

      for (int i = 0; i < 64; ++i)
      {
         for (int j = 0; j < 64; ++j)
         {
            final int c = ((i & 0x8)==0 ^ (j & 0x8)==0)? 0 : 1;
            image[i][j] = (byte) c;
         }
      }
      return image;
   }


   public static byte[] makeImage64()
   {
      final byte[] image = new byte[64 * 64 * 3];

      for (int i = 0; i < 64; ++i)
      {
         for (int j = 0; j < 64; ++j)
         {
          //final int c = (i | j) % 255;
            final int c = ((i & 0x8)==0 ^ (j & 0x8)==0) ? 255 : 0;
            image[i*3*64 + j*3 + 0] = (byte)c;  // r
            image[i*3*64 + j*3 + 1] = (byte)c;  // g
            image[i*3*64 + j*3 + 2] = (byte)c;  // b
         }
      }
      return image;
   }


   // A Sierpinski triangle patter that is
   // powerOfTwo pixels high by powerOfTwo pixels wide.
   public static byte[] sierpinskiImage(final int powerOfTwo)
   {
      final byte[] image = new byte[powerOfTwo * powerOfTwo * 3];

      for (int i = 0; i < powerOfTwo; ++i)
      {
         for (int j = 0; j < powerOfTwo; ++j)
         {
            final int c = (i | j) % 255;
            image[i*3*powerOfTwo + j*3 + 0] = (byte)c;  // r
            image[i*3*powerOfTwo + j*3 + 1] = (byte)c;  // g
            image[i*3*powerOfTwo + j*3 + 2] = (byte)c;  // b
         }
      }
      return image;
   }


   // An 8-by-8 checkboard patter that is
   // powerOfTwo pixels high by powerOfTwo pixels wide.
   public static byte[] makeImage(final int powerOfTwo)
   {
      final byte[] image = new byte[powerOfTwo * powerOfTwo * 3];

      int digit = powerOfTwo / 8;

      for (int i = 0; i < powerOfTwo; ++i)
      {
         for (int j = 0; j < powerOfTwo; ++j)
         {
          //final int c = (i | j) % 255;
            final int c = ((i & digit)==0 ^ (j & digit)==0) ? 255 : 0;
            image[i*3*powerOfTwo + j*3 + 0] = (byte)c;  // r
            image[i*3*powerOfTwo + j*3 + 1] = (byte)c;  // g
            image[i*3*powerOfTwo + j*3 + 2] = (byte)c;  // b
         }
      }
      return image;
   }


   // An N-by-N checkboard patter that is
   // powerOfTwo pixels high by powerOfTwo pixels wide.
   // N must divide powerOfTwo.
   public static byte[] makeImage(final int powerOfTwo, final int N)
   {
      final byte[] image = new byte[powerOfTwo * powerOfTwo * 3];

      int digit = powerOfTwo / N;

      for (int i = 0; i < powerOfTwo; ++i)
      {
         for (int j = 0; j < powerOfTwo; ++j)
         {
          //final int c = (i | j) % 255;
            final int c = ((i & digit)==0 ^ (j & digit)==0) ? 255 : 0;
            image[i*3*powerOfTwo + j*3 + 0] = (byte)c;  // r
            image[i*3*powerOfTwo + j*3 + 1] = (byte)c;  // g
            image[i*3*powerOfTwo + j*3 + 2] = (byte)c;  // b
         }
      }
      return image;
   }


   // An N-by-M checkboard patter that is
   // powerOfTwo pixels high by powerOfTwo pixels wide.
   // N and M must divide powerOfTwo.
   public static byte[] makeImage(final int powerOfTwo,
                                  final int N,
                                  final int M)
   {
      final byte[] image = new byte[powerOfTwo * powerOfTwo * 3];

      int digit1 = powerOfTwo / N;
      int digit2 = powerOfTwo / M;

      for (int i = 0; i < powerOfTwo; ++i)
      {
         for (int j = 0; j < powerOfTwo; ++j)
         {
          //final int c = (i | j) % 255;
            final int c = ((i & digit1)==0 ^ (j & digit2)==0) ? 255 : 0;
            image[i*3*powerOfTwo + j*3 + 0] = (byte)c;  // r
            image[i*3*powerOfTwo + j*3 + 1] = (byte)c;  // g
            image[i*3*powerOfTwo + j*3 + 2] = (byte)c;  // b
         }
      }
      return image;
   }
}
