/*
https://www.cs.unm.edu/~angel/BOOK/PRIMER/THIRD_EDITION/PROGRAMS/CHAPTER08/cubetex.c
*/

import com.jogamp.opengl.GLEventListener;
import com.jogamp.opengl.GLAutoDrawable;
import com.jogamp.opengl.GLCapabilities;
import com.jogamp.opengl.GLProfile;
import com.jogamp.opengl.awt.GLJPanel;
import com.jogamp.opengl.GL2;
import com.jogamp.opengl.glu.GLU;
import com.jogamp.opengl.util.gl2.GLUT;
import com.jogamp.opengl.util.FPSAnimator;

import java.awt.event.*;
import javax.swing.JFrame;
import java.nio.ByteBuffer;
import java.awt.image.*;
import java.io.*;
import javax.imageio.*;

/**

*/
public class CubeTexture_v3 implements GLEventListener, KeyListener
{
   private final int[] texName = new int[6];

   private double theta[] = {0.0, 0.0, 0.0};
   private int axis = 2;

   private double[][] vertices = {{-1.0, -1.0, -1.0},
                                  { 1.0, -1.0, -1.0},
                                  { 1.0,  1.0, -1.0},
                                  {-1.0,  1.0, -1.0},
                                  {-1.0, -1.0,  1.0},
                                  { 1.0, -1.0,  1.0},
                                  { 1.0,  1.0,  1.0},
                                  {-1.0,  1.0,  1.0}};

   private double[][] colors = {{0.0, 0.0, 0.0, 0.5},
                                {1.0, 0.0, 0.0, 0.5},
                                {1.0, 1.0, 0.0, 0.5},
                                {0.0, 1.0, 0.0, 0.5},
                                {0.0, 0.0, 1.0, 0.5},
                                {1.0, 0.0, 1.0, 0.5},
                                {1.0, 1.0, 1.0, 0.5},
                                {0.0, 1.0, 1.0, 0.5}};

   private double[][] texcoords = {{0.0, 0.0},
                                   {0.0, 1.0},
                                   {1.0, 1.0},
                                   {1.0, 0.0}};

   public CubeTexture_v3()
   {
      // Get a capabilities object from a GL2 profile.
      final GLProfile glProfile = GLProfile.get(GLProfile.GL2);
      final GLCapabilities glCapabilities = new GLCapabilities(glProfile);

      // Use it to create a GLJPanel.
      final GLJPanel glDrawable = new GLJPanel(glCapabilities);
      glDrawable.setPreferredSize(new java.awt.Dimension(500, 500));

      // Connect the GLJPanel with our GLEventListener implementation.
      glDrawable.addGLEventListener( this );

      // Add the GLJPanel to a JFrame.
      final JFrame jFrame = new JFrame ("Cube Texture v3");
      jFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      jFrame.getContentPane().add(glDrawable);
      jFrame.pack();
      jFrame.setLocationRelativeTo(null);
      jFrame.setVisible(true);

      glDrawable.addKeyListener(this);

      // Create an animation thread.
      final FPSAnimator animator = new FPSAnimator(glDrawable, 24, true);
      animator.start();

      System.out.println("Use the 'x', 'y', 'z' keys to change the axis of rotation.");
   }


   // Draw a polygon via list of vertices.
   private void polygon(GL2 gl, int a, int b, int c , int d)
   {
      gl.glBegin(gl.GL_QUADS);
         //gl.glColor4dv(colors[a], 0);
         gl.glTexCoord2dv(texcoords[0], 0);
         gl.glVertex3dv(vertices[a], 0);

         //gl.glColor4dv(colors[b], 0);
         gl.glTexCoord2dv(texcoords[1], 0);
         gl.glVertex3dv(vertices[b], 0);

         //gl.glColor4dv(colors[c], 0);
         gl.glTexCoord2dv(texcoords[2], 0);
         gl.glVertex3dv(vertices[c], 0);

         //gl.glColor4dv(colors[d], 0);
         gl.glTexCoord2dv(texcoords[3], 0);
         gl.glVertex3dv(vertices[d], 0);
      gl.glEnd();
   }


   // Map vertices to faces.
   private void colorcube(GL2 gl)
   {
      gl.glBindTexture(gl.GL_TEXTURE_2D, texName[0]);
      polygon(gl, 0, 3, 2, 1);

      gl.glBindTexture(gl.GL_TEXTURE_2D, texName[1]);
      polygon(gl, 2, 3, 7, 6);

      gl.glBindTexture(gl.GL_TEXTURE_2D, texName[2]);
      polygon(gl, 3, 0, 4, 7);

      gl.glBindTexture(gl.GL_TEXTURE_2D, texName[3]);
      polygon(gl, 1, 2, 6, 5);

      gl.glBindTexture(gl.GL_TEXTURE_2D, texName[4]);
      polygon(gl, 4, 5, 6, 7);

      gl.glBindTexture(gl.GL_TEXTURE_2D, texName[5]);
      polygon(gl, 5, 4, 0, 1);
   }


   // Implement the GLEventListener interface (four methods).
   @Override
   public void init(GLAutoDrawable drawable)
   {
      final GL2 gl = drawable.getGL().getGL2();

      gl.glClearColor(0.8f, 0.8f, 0.8f, 1.0f);
    //gl.glShadeModel(gl.GL_FLAT);
      gl.glEnable(gl.GL_DEPTH_TEST);

      gl.glGenTextures(6, texName, 0);

      loadTexture(gl, 0, "textures/uv_grid_opengl.jpg");
      loadTexture(gl, 1, "textures/brick001.jpg");
      loadTexture(gl, 2, "textures/brick_diffuse.jpg");
      loadTexture(gl, 3, "textures/marble.jpg");
      loadTexture(gl, 4, "textures/metal003.jpg");
      loadTexture(gl, 5, "textures/mandelbrot.jpeg");
   }


   private void loadTexture(final GL2 gl, final int index, final String fileName)
   {
      try
      {
         final File f = new File(fileName);
         final BufferedImage bufimg = ImageIO.read(f);
         final int imageWidth = bufimg.getWidth();
         final int imageHeight = bufimg.getHeight();
         final Raster raster = bufimg.getData();
         final byte[] image = ((DataBufferByte)raster.getDataBuffer()).getData();

         gl.glBindTexture(gl.GL_TEXTURE_2D, texName[index]);
         gl.glTexParameteri(gl.GL_TEXTURE_2D, gl.GL_TEXTURE_MIN_FILTER, gl.GL_NEAREST);
         gl.glTexParameteri(gl.GL_TEXTURE_2D, gl.GL_TEXTURE_MAG_FILTER, gl.GL_NEAREST);
         gl.glTexImage2D(gl.GL_TEXTURE_2D, 0, gl.GL_RGB, imageWidth, imageHeight,
                                           0, gl.GL_BGR,// Notice the change in color order!
                                           gl.GL_UNSIGNED_BYTE,
                                           ByteBuffer.wrap(image));
      }
      catch (IOException e)
      {
         System.err.printf("ERROR! Could not open %s\n", fileName);
         e.printStackTrace(System.err);
         System.exit(-1);
      }
   }


   @Override
   public void display(GLAutoDrawable drawable)
   {
      final GL2 gl = drawable.getGL().getGL2();

      gl.glClear(gl.GL_COLOR_BUFFER_BIT | gl.GL_DEPTH_BUFFER_BIT);

      gl.glEnable(gl.GL_TEXTURE_2D);
    //gl.glTexEnvf(gl.GL_TEXTURE_ENV, gl.GL_TEXTURE_ENV_MODE, gl.GL_DECAL);
    //gl.glTexEnvf(gl.GL_TEXTURE_ENV, gl.GL_TEXTURE_ENV_MODE, gl.GL_REPLACE);

      gl.glLoadIdentity();
      gl.glRotated(theta[0], 1.0, 0.0, 0.0);
      gl.glRotated(theta[1], 0.0, 1.0, 0.0);
      gl.glRotated(theta[2], 0.0, 0.0, 1.0);
      colorcube(gl);

      theta[axis] += 2.0;
      if( theta[axis] > 360.0 ) theta[axis] -= 360.0;
   }


   @Override
   public void reshape(GLAutoDrawable drawable, int x, int y, int w, int h)
   {
      final GL2 gl = drawable.getGL().getGL2();

      gl.glViewport(0, 0, w, h);
      gl.glMatrixMode(gl.GL_PROJECTION);
      gl.glLoadIdentity();
      if (w <= h)
         gl.glOrtho(-1.5, 1.5, -1.5*(float)h/(float)w,
                     1.5*(float)h/(float)w, -10.0, 10.0);
      else
         gl.glOrtho(-1.5*(float)w/(float)h,
                     1.5*(float)w/(float)h, -1.5, 1.5, -10.0, 10.0);
      gl.glMatrixMode(gl.GL_MODELVIEW);
      gl.glLoadIdentity();
   }


   @Override
   public void dispose(GLAutoDrawable drawable)
   {
   }


   // implement the KeyListener interface (three methods)
   @Override public void keyPressed(KeyEvent e){}
   @Override public void keyReleased(KeyEvent e){}
   @Override public void keyTyped(KeyEvent e)
   {
      final char c = e.getKeyChar();
      if ('x' == c)
      {
         axis = 0;
      }
      else if ('y' == c)
      {
         axis = 1;
      }
      else if ('z' == c)
      {
         axis = 2;
      }
   }


   public static void main(String[] args)
   {
      javax.swing.SwingUtilities.invokeLater(
         () -> new CubeTexture_v3()
      );
   }
}
