/*
   http://www.glprogramming.com/red/chapter09.html
*/

import com.jogamp.opengl.GLEventListener;
import com.jogamp.opengl.GLAutoDrawable;
import com.jogamp.opengl.GLCapabilities;
import com.jogamp.opengl.GLProfile;
import com.jogamp.opengl.awt.GLJPanel;
import com.jogamp.opengl.GL2;
import com.jogamp.opengl.glu.GLU;
import com.jogamp.opengl.util.gl2.GLUT;

import javax.swing.JFrame;
import java.nio.ByteBuffer;
import java.awt.image.*;
import java.io.*;
import javax.imageio.*;

/**

*/
public class ImageTexture implements GLEventListener
{
   private final int[] texName = new int[1];

   public ImageTexture()
   {
      // Get a capabilities object from a GL2 profile.
      final GLProfile glProfile = GLProfile.get(GLProfile.GL2);
      final GLCapabilities glCapabilities = new GLCapabilities(glProfile);

      // Use it to create a GLJPanel.
      final GLJPanel glDrawable = new GLJPanel(glCapabilities);
      glDrawable.setPreferredSize(new java.awt.Dimension(500, 500));

      // Connect the GLJPanel with our GLEventListener implementation.
      glDrawable.addGLEventListener( this );

      // Add the GLJPanel to a JFrame.
      final JFrame jFrame = new JFrame ("Image Texture");
      jFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      jFrame.getContentPane().add(glDrawable);
      jFrame.pack();
      jFrame.setLocationRelativeTo(null);
      jFrame.setVisible(true);
   }


   // Implement the GLEventListener interface (four methods).
   @Override
   public void init(GLAutoDrawable drawable)
   {
      final GL2 gl = drawable.getGL().getGL2();

      final String fileName = "textures/uv_grid_opengl.jpg";
    //final String fileName = "textures/brick001.jpg";

      BufferedImage bufimg = null;
      try
      {
         final File f = new File(fileName);
         bufimg = ImageIO.read(f);
      }
      catch (IOException e)
      {
         System.err.printf("ERROR! Could not open %s\n", fileName);
         e.printStackTrace(System.err);
         System.exit(-1);
      }

      final int imageWidth = bufimg.getWidth();
      final int imageHeight = bufimg.getHeight();
      final Raster raster = bufimg.getData();
      final byte[] image = ((DataBufferByte)raster.getDataBuffer()).getData();

      gl.glClearColor(0.8f, 0.8f, 0.8f, 1.0f);
    //gl.glShadeModel(gl.GL_FLAT);
    //gl.glEnable(gl.GL_DEPTH_TEST);

      gl.glGenTextures(1, texName, 0);

      gl.glBindTexture(gl.GL_TEXTURE_2D, texName[0]);
    //gl.glTexParameteri(gl.GL_TEXTURE_2D, gl.GL_TEXTURE_WRAP_S,     gl.GL_REPEAT);
    //gl.glTexParameteri(gl.GL_TEXTURE_2D, gl.GL_TEXTURE_WRAP_T,     gl.GL_REPEAT);
      gl.glTexParameteri(gl.GL_TEXTURE_2D, gl.GL_TEXTURE_MIN_FILTER, gl.GL_NEAREST);
      gl.glTexParameteri(gl.GL_TEXTURE_2D, gl.GL_TEXTURE_MAG_FILTER, gl.GL_NEAREST);

      gl.glTexImage2D(gl.GL_TEXTURE_2D,
                      0,
                      gl.GL_RGB,
                      imageWidth,
                      imageHeight,
                      0,
                      gl.GL_BGR,  // Notice the change in color order!
                      gl.GL_UNSIGNED_BYTE,
                      ByteBuffer.wrap(image));
   }


   @Override
   public void display(GLAutoDrawable drawable)
   {
      final GL2 gl = drawable.getGL().getGL2();

      gl.glClear(gl.GL_COLOR_BUFFER_BIT | gl.GL_DEPTH_BUFFER_BIT);

      gl.glEnable(gl.GL_TEXTURE_2D); /** try commenting this out **/

    //gl.glTexEnvf(gl.GL_TEXTURE_ENV, gl.GL_TEXTURE_ENV_MODE, gl.GL_DECAL);
    //gl.glTexEnvf(gl.GL_TEXTURE_ENV, gl.GL_TEXTURE_ENV_MODE, gl.GL_REPLACE);

      gl.glBindTexture(gl.GL_TEXTURE_2D, texName[0]); /** try commenting this out **/

      gl.glBegin(gl.GL_QUADS);
        // The texture and vertex coordinates are vertically flipped!
        gl.glTexCoord2d(0.0, 1.0); gl.glVertex3d(-2.0, -1.0, 0.0);
        gl.glTexCoord2d(0.0, 0.0); gl.glVertex3d(-2.0,  1.0, 0.0);
        gl.glTexCoord2d(1.0, 0.0); gl.glVertex3d( 0.0,  1.0, 0.0);
        gl.glTexCoord2d(1.0, 1.0); gl.glVertex3d( 0.0, -1.0, 0.0);

        gl.glTexCoord2d(0.0, 1.0); gl.glVertex3d(1.0,     -1.0,  0.0);
        gl.glTexCoord2d(0.0, 0.0); gl.glVertex3d(1.0,      1.0,  0.0);
        gl.glTexCoord2d(1.0, 0.0); gl.glVertex3d(2.41421,  1.0, -1.41421);
        gl.glTexCoord2d(1.0, 1.0); gl.glVertex3d(2.41421, -1.0, -1.41421);
      gl.glEnd();
      gl.glFlush();
      gl.glDisable(gl.GL_TEXTURE_2D);
   }


   @Override
   public void reshape(GLAutoDrawable drawable, int x, int y, int w, int h)
   {
      final GL2 gl = drawable.getGL().getGL2();
      final GLU glu = new GLU();  // GL Utilities

      gl.glViewport(0, 0, w, h);
      gl.glMatrixMode(gl.GL_PROJECTION);
      gl.glLoadIdentity();
      glu.gluPerspective(60.0, (float)w/(float)h, 1.0, 30.0);

      gl.glMatrixMode(gl.GL_MODELVIEW);
      gl.glLoadIdentity();
      gl.glTranslated(0.0, 0.0, -3.6);
   }


   @Override
   public void dispose(GLAutoDrawable drawable)
   {
   }

   public static void main(String[] args)
   {
      javax.swing.SwingUtilities.invokeLater(
         () -> new ImageTexture()
      );
   }
}
