/*

*/

import com.jogamp.opengl.GLEventListener;
import com.jogamp.opengl.GLAutoDrawable;
import com.jogamp.opengl.GLCapabilities;
import com.jogamp.opengl.GLProfile;
import com.jogamp.opengl.awt.GLJPanel;
import com.jogamp.opengl.GL2;
import com.jogamp.opengl.glu.GLU;
import com.jogamp.opengl.util.gl2.GLUT;

import javax.swing.JFrame;
import java.nio.ByteBuffer;
import java.awt.image.*;
import java.io.*;
import javax.imageio.*;

/**

*/
public class TwoTriangles_v1 implements GLEventListener
{
   private final int[] texName = new int[3];

   public TwoTriangles_v1()
   {
      // Get a capabilities object from a GL2 profile.
      final GLProfile glProfile = GLProfile.get(GLProfile.GL2);
      final GLCapabilities glCapabilities = new GLCapabilities(glProfile);

      // Use it to create a GLJPanel.
      final GLJPanel glDrawable = new GLJPanel(glCapabilities);
      glDrawable.setPreferredSize(new java.awt.Dimension(500, 500));

      // Connect the GLJPanel with our GLEventListener implementation.
      glDrawable.addGLEventListener( this );

      // Add the GLJPanel to a JFrame.
      final JFrame jFrame = new JFrame ("Two Triangles v1");
      jFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      jFrame.getContentPane().add(glDrawable);
      jFrame.pack();
      jFrame.setLocationRelativeTo(null);
      jFrame.setVisible(true);
   }


   // Implement the GLEventListener interface (four methods).
   @Override
   public void init(GLAutoDrawable drawable)
   {
      final GL2 gl = drawable.getGL().getGL2();

      gl.glClearColor(0.8f, 0.8f, 0.8f, 1.0f);
    //gl.glShadeModel(gl.GL_FLAT);
    //gl.glEnable(gl.GL_DEPTH_TEST);

      gl.glGenTextures(3, texName, 0);

      gl.glBindTexture(gl.GL_TEXTURE_2D, texName[0]);
      gl.glTexParameteri(gl.GL_TEXTURE_2D, gl.GL_TEXTURE_MIN_FILTER, gl.GL_NEAREST);
      gl.glTexParameteri(gl.GL_TEXTURE_2D, gl.GL_TEXTURE_MAG_FILTER, gl.GL_NEAREST);
      final int imageWidth = 1024;
      final int imageHeight = imageWidth;
      final byte[] image = CheckerPattern.makeImage(imageWidth);
      gl.glTexImage2D(gl.GL_TEXTURE_2D, 0, gl.GL_RGB, imageWidth, imageHeight,
                                        0, gl.GL_RGB,
                                        gl.GL_UNSIGNED_BYTE,
                                        ByteBuffer.wrap(image));

      loadTexture(gl, 1, "textures/brick001.jpg");
      loadTexture(gl, 2, "textures/mandelbrot.jpeg");
   }


   private void loadTexture(final GL2 gl, final int index, final String fileName)
   {
      try
      {
         final File f = new File(fileName);
         final BufferedImage bufimg = ImageIO.read(f);
         final int imageWidth = bufimg.getWidth();
         final int imageHeight = bufimg.getHeight();
         final Raster raster = bufimg.getData();
         final byte[] image = ((DataBufferByte)raster.getDataBuffer()).getData();

         gl.glBindTexture(gl.GL_TEXTURE_2D, texName[index]);
         gl.glTexParameteri(gl.GL_TEXTURE_2D, gl.GL_TEXTURE_MIN_FILTER, gl.GL_NEAREST);
         gl.glTexParameteri(gl.GL_TEXTURE_2D, gl.GL_TEXTURE_MAG_FILTER, gl.GL_NEAREST);
         gl.glTexImage2D(gl.GL_TEXTURE_2D, 0, gl.GL_RGB, imageWidth, imageHeight,
                                           0, gl.GL_BGR,// Notice the change in color order!
                                           gl.GL_UNSIGNED_BYTE,
                                           ByteBuffer.wrap(image));
      }
      catch (IOException e)
      {
         System.err.printf("ERROR! Could not open %s\n", fileName);
         e.printStackTrace(System.err);
         System.exit(-1);
      }
   }


   @Override
   public void display(GLAutoDrawable drawable)
   {
      final GL2 gl = drawable.getGL().getGL2();

      gl.glClear(gl.GL_COLOR_BUFFER_BIT | gl.GL_DEPTH_BUFFER_BIT);

      gl.glEnable(gl.GL_TEXTURE_2D);

      gl.glBindTexture(gl.GL_TEXTURE_2D, texName[1]);
    //gl.glBindTexture(gl.GL_TEXTURE_2D, texName[2]);
      gl.glTexParameterf(gl.GL_TEXTURE_2D, gl.GL_TEXTURE_WRAP_S, gl.GL_REPEAT);
      gl.glTexParameterf(gl.GL_TEXTURE_2D, gl.GL_TEXTURE_WRAP_T, gl.GL_MIRRORED_REPEAT);
      gl.glBegin(gl.GL_TRIANGLES);
        gl.glBindTexture(gl.GL_TEXTURE_2D, texName[0]);
        gl.glTexCoord2d(0.0, 0.0); gl.glVertex3d(-1.0, -1.0, 0.0);
        gl.glTexCoord2d(0.0, 2.0); gl.glVertex3d(-1.0,  1.0, 0.0);
        gl.glTexCoord2d(3.0, 2.0); gl.glVertex3d( 1.0,  1.0, 0.0);
      gl.glEnd();

      gl.glBindTexture(gl.GL_TEXTURE_2D, texName[2]);
      gl.glBegin(gl.GL_TRIANGLES);
        gl.glTexCoord2d(0.0, 0.0); gl.glVertex3d(-1.0, -1.0, 0.0);
        gl.glTexCoord2d(3.0, 2.0); gl.glVertex3d( 1.0,  1.0, 0.0);
        gl.glTexCoord2d(3.0, 0.0); gl.glVertex3d( 1.0, -1.0, 0.0);
      gl.glEnd();
      gl.glFlush();
      gl.glDisable(gl.GL_TEXTURE_2D);
   }


   @Override
   public void reshape(GLAutoDrawable drawable, int x, int y, int w, int h)
   {
      final GL2 gl = drawable.getGL().getGL2();
      final GLU glu = new GLU();  // GL Utilities

      gl.glViewport(0, 0, w, h);
      gl.glMatrixMode(gl.GL_PROJECTION);
      gl.glLoadIdentity();
      glu.gluPerspective(60.0, (float)w/(float)h, 1.0, 30.0);

      gl.glMatrixMode(gl.GL_MODELVIEW);
      gl.glLoadIdentity();
      gl.glTranslated(0.0, 0.0, -3.6);
   }


   @Override
   public void dispose(GLAutoDrawable drawable)
   {
   }

   public static void main(String[] args)
   {
      javax.swing.SwingUtilities.invokeLater(
         () -> new TwoTriangles_v1()
      );
   }
}
